include "Model100.inc"

ext WaitTimer_

LoPort equ $bc
HighPort equ $bd

tempo equ 220


dseg
SoundPtr:
    defw 0
SoundLen:
    defb 0
time:
    defw 0


cseg
InitSound_: public InitSound_
    push psw | push h
        di
            lxi h,0
            shld time
            shld SoundPtr
        ei
    pop h | pop psw
ret


ToneOn:
; in
;   hl : cycle
    push psw
        mov a,l | out LoPort
        mov a,h | ori $40 | out HighPort
        mvi a,$c3 | out $b8
        in PORTB
            ani not $04
            ori $20
        out PORTB
    pop psw
ret

ToneOff:
    push psw
        in PORTB
            ani not $20
            ori 04
        out PORTB
    pop psw
ret


SoundHandler: public SoundHandler
    push psw | push h | push d | push b
        lhld time
        lxi d,-tempo
        dad d
        if nc
            push h
                channel0:
                lhld SoundPtr
                mov a,l | ora h
                if nz
                    lda SoundLen
                    dcr a
                    sta SoundLen
                    if z
                        continue0:
                        mov a,m | inx h
                        ora a
                        if z
                            ; end0
                            lxi h,0
                            shld SoundPtr
                            offch0:
                            call ToneOff
                            jmp channelend
                        endif
                        sta SoundLen
                        mov a,m | inx h
                        shld SoundPtr
                        ora a
                        jz offch0
                        dcr a
                        add a
                        mov e,a
                        mvi d,0
                        lxi h,cycles
                        dad d
                        mov a,m | inx h
                        mov h,m
                        mov l,a
                        call ToneOn
                    endif
                else
                    call ToneOff
                endif
                channelend:
            pop h
            lxi d,600/2
            dad d
        endif
        shld time
    pop b | pop d | pop h | pop psw
ret


Melody:
    push psw
        di
            call ToneOff
            shld SoundPtr
            mvi a,1 | sta SoundLen
        ei
    pop psw
ret

MelodyWait:
    call Melody
    push psw
        do
            lhld SoundPtr
            mov a,l | ora h
        while nz | wend
    pop psw
ret 


loose_notes:
    defb 1,A3, 0
Sound_Loose_: public Sound_Loose_
    push h
        lxi h,loose_notes
        call MelodyWait
    pop h
ret


hit_notes:
    defb 1,F4, 1,G4, 1,A4, 1,B4, 1,C5, 1,D5, 1,E5, 1,F5
    defb 0 
Sound_Hit_: public Sound_Hit_
    push h
        lxi h,hit_notes
        call Melody
    pop h
ret


beep_notes:
    defb 1,A4, 0
Sound_Beep_: public Sound_Beep_
Sound_Get_: public Sound_Get_
    push h
        lxi h,beep_notes
        call MelodyWait
    pop h
ret


start_notes:
    defb 5,C4,1,0, 5,G4,1,0, 5,E4,1,0, 5,G4,1,0, 23,C5,1,0
    defb 0
Sound_Start_: public Sound_Start_
    push h
        lxi h,start_notes
        call MelodyWait
    pop h
ret


clear_notes:
	defb 5,C4,1,0, 5,E4,1,0, 5,G4,1,0, 5,D4,1,0, 5,F4,1,0, 5,A4,1,0, 5,E4,1,0, 5,G4,1,0
	defb 5,B4,1,0, 17,C5,1,0
    defb 0
Sound_Clear_: public Sound_Clear_
    push h
        lxi h,clear_notes
        call MelodyWait
    pop h
ret


over_notes:
	defb 5,C5,1,0, 5,G4,1,0, 5,E4,1,0, 5,C5,1,0, 5,B4,1,0, 5,G4,1,0, 5,E4,1,0, 5,B4,1,0
	defb 11,A4,1,0, 11,B4,1,0, 23,C5,1,0
    defb 0
Sound_GameOver_: public Sound_GameOver_
    push h
        lxi h,over_notes
        call MelodyWait
    pop h
ret


G3 equ 1
G3S equ 2
A3 equ 3
A3S equ 4
B3 equ 5
C4 equ 6
C4S equ 7
D4 equ 8
D4S equ 9
E4 equ 10
F4 equ 11
F4S equ 12
G4 equ 13
G4S equ 14
A4 equ 15
A4S equ 16
B4 equ 17
C5 equ 18
C5S equ 19
D5 equ 20
D5S equ 21
E5 equ 22
F5 equ 23
F5S equ 24
G5 equ 25

cycles:
defw 12538 ; G3
defw 11835 ; G#3
defw 11170 ; A3
defw 10543 ; A#3
defw 9952 ; B3
defw 9393 ; C4
defw 8866 ; C#4
defw 8368 ; D4
defw 7899 ; D#4
defw 7455 ; E4
defw 7037 ; F4
defw 6642 ; F#4
defw 6269 ; G4
defw 5917 ; G#4
defw 5585 ; A4
defw 5271 ; A#4
defw 4976 ; B4
defw 4696 ; C5
defw 4433 ; C#5
defw 4184 ; D5
defw 3949 ; D#5
defw 3727 ; E5
defw 3518 ; F5
defw 3321 ; F#5
defw 3134 ; G5
